# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING
from typing_extensions import Self

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import HybridComputeManagementClientConfiguration
from .operations import (
    ExtensionMetadataOperations,
    GatewaysOperations,
    HybridComputeManagementClientOperationsMixin,
    LicenseProfilesOperations,
    LicensesOperations,
    MachineExtensionsOperations,
    MachineRunCommandsOperations,
    MachinesOperations,
    NetworkProfileOperations,
    NetworkSecurityPerimeterConfigurationsOperations,
    Operations,
    PrivateEndpointConnectionsOperations,
    PrivateLinkResourcesOperations,
    PrivateLinkScopesOperations,
    SettingsOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class HybridComputeManagementClient(
    HybridComputeManagementClientOperationsMixin
):  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """The Hybrid Compute Management Client.

    :ivar licenses: LicensesOperations operations
    :vartype licenses: azure.mgmt.hybridcompute.aio.operations.LicensesOperations
    :ivar machines: MachinesOperations operations
    :vartype machines: azure.mgmt.hybridcompute.aio.operations.MachinesOperations
    :ivar license_profiles: LicenseProfilesOperations operations
    :vartype license_profiles: azure.mgmt.hybridcompute.aio.operations.LicenseProfilesOperations
    :ivar machine_extensions: MachineExtensionsOperations operations
    :vartype machine_extensions:
     azure.mgmt.hybridcompute.aio.operations.MachineExtensionsOperations
    :ivar extension_metadata: ExtensionMetadataOperations operations
    :vartype extension_metadata:
     azure.mgmt.hybridcompute.aio.operations.ExtensionMetadataOperations
    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.hybridcompute.aio.operations.Operations
    :ivar network_profile: NetworkProfileOperations operations
    :vartype network_profile: azure.mgmt.hybridcompute.aio.operations.NetworkProfileOperations
    :ivar machine_run_commands: MachineRunCommandsOperations operations
    :vartype machine_run_commands:
     azure.mgmt.hybridcompute.aio.operations.MachineRunCommandsOperations
    :ivar gateways: GatewaysOperations operations
    :vartype gateways: azure.mgmt.hybridcompute.aio.operations.GatewaysOperations
    :ivar settings: SettingsOperations operations
    :vartype settings: azure.mgmt.hybridcompute.aio.operations.SettingsOperations
    :ivar private_link_scopes: PrivateLinkScopesOperations operations
    :vartype private_link_scopes:
     azure.mgmt.hybridcompute.aio.operations.PrivateLinkScopesOperations
    :ivar private_link_resources: PrivateLinkResourcesOperations operations
    :vartype private_link_resources:
     azure.mgmt.hybridcompute.aio.operations.PrivateLinkResourcesOperations
    :ivar private_endpoint_connections: PrivateEndpointConnectionsOperations operations
    :vartype private_endpoint_connections:
     azure.mgmt.hybridcompute.aio.operations.PrivateEndpointConnectionsOperations
    :ivar network_security_perimeter_configurations:
     NetworkSecurityPerimeterConfigurationsOperations operations
    :vartype network_security_perimeter_configurations:
     azure.mgmt.hybridcompute.aio.operations.NetworkSecurityPerimeterConfigurationsOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2024-07-31-preview". Note that overriding
     this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = HybridComputeManagementClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.licenses = LicensesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.machines = MachinesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.license_profiles = LicenseProfilesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.machine_extensions = MachineExtensionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.extension_metadata = ExtensionMetadataOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.network_profile = NetworkProfileOperations(self._client, self._config, self._serialize, self._deserialize)
        self.machine_run_commands = MachineRunCommandsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.gateways = GatewaysOperations(self._client, self._config, self._serialize, self._deserialize)
        self.settings = SettingsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.private_link_scopes = PrivateLinkScopesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_link_resources = PrivateLinkResourcesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.private_endpoint_connections = PrivateEndpointConnectionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.network_security_perimeter_configurations = NetworkSecurityPerimeterConfigurationsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> Self:
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
